@@ -1,25 +1,18 @@
-import mdtraj
-import numpy as np
-
 import os
-
-from simtk.openmm import app
-from simtk.openmm.app import PDBFile
-from simtk import openmm as mm
-from simtk.openmm import unit
+import pickle
+import re
+import shutil
+from platform import uname
 from sys import stdout
-
 from tempfile import mkdtemp
 
-from .utils import *
-
-import pickle
-
-import shutil
+import mdtraj
+from simtk import openmm as mm
+from simtk.openmm import app
+from simtk.openmm import unit
 
 from .custom_reporters import FESReporter
-
-from platform import uname
+from .utils import *
 
 
 class MDSimulation():
@@ -175,9 +168,12 @@ class MDSimulation():
         if box_vectors is not None:
             print('using box vectors specified at input')
             self.box_vectors = box_vectors
-        else:
-            print('getting box vectors from parmed')
+        elif parmed_structure.box_vectors is not None:
+            print('getting box vectors from parmed {}'.format(parmed_structure.box_vectors))
             self.box_vectors = parmed_structure.box_vectors
+        else:
+            print('did not find any box vectors')
+            self.box_vectors = None
 
         self.centering=centering
         self.cwd = cwd
@@ -215,12 +211,26 @@ class MDSimulation():
         self.progress_out = progress_out
         self.custom_forces = {}
 
-        # if they're still none this should be an implicit or vacuum sim
+        if self.box_vectors == None:
+            self.wrap_state_coords = False
+        else:
+            self.wrap_state_coords = True
+
+        # if box v's are still none this should be an implicit or vacuum sim
         if self.box_vectors == None and self.implicit_solvent_model == None:
             print('Warning! No box vectors found and no implicit model, proceeding with vacuum simulation')
 
         # run sanity checks
 
+        #dont use PME with implicit
+        if self.implicit_solvent_model is not None and self.nonbonded_method == 'PME':
+            raise ValueError('implicit solvent can not be used with PME')
+
+        #if implicit used we ignore box vectors and warn the user
+        if self.implicit_solvent_model is not None and self.box_vectors is not None:
+            print('Warning, found box vectors but running implicit solvent, ignoring box vectors')
+            self.box_vectors = None
+
         # make sure the parmed has all the params
         if len([i.type for i in parmed_structure.bonds if i.type == None]) > 0:
             raise ValueError('Parmed object does not have all parameters')
@@ -326,11 +336,15 @@ class MDSimulation():
             raise ValueError('Unknown implicit solvent model {}'.format(implicit_solvent_model))
 
         implicit_model_object = eval("app.%s" % implicit_solvent_model)
-        nonbonded_method_object = eval("app.%s" % nonbonded_method)
-        implicit_force = self.parmed_structure.omm_gbsa_force(implicit_model_object,
-                                                              temperature=self.temperature,
-                                                              nonbondedMethod=nonbonded_method_object,
-                                                              nonbondedCutoff=nonbonded_cutoff)
+        if nonbonded_method is not None:
+            nonbonded_method_object = eval("app.%s" % nonbonded_method)
+            implicit_force = self.parmed_structure.omm_gbsa_force(implicit_model_object,
+                                                                  temperature=self.temperature,
+                                                                  nonbondedMethod=nonbonded_method_object,
+                                                                  nonbondedCutoff=nonbonded_cutoff)
+        else:
+            implicit_force = self.parmed_structure.omm_gbsa_force(implicit_model_object,
+                                                                  temperature=self.temperature)
 
         self.custom_forces['implicit_solvent'] = self.system.addForce(implicit_force)
 
@@ -430,30 +444,38 @@ class MDSimulation():
             print('did not set any constraints')
             constraints_object = None
 
-        nonbonded_method_object = eval("app.%s" % self.nonbonded_method)
-
-
-        self.system = self.parmed_structure.createSystem(nonbondedMethod=nonbonded_method_object,
-                                                         nonbondedCutoff=self.nonbonded_cutoff,
-                                                         constraints=constraints_object,
-                                                         removeCMMotion=False,
-                                                         hydrogenMass=4.0 * unit.amu if self.hmr else None)
+        if self.nonbonded_method is not None:
+            nonbonded_method_object = eval("app.%s" % self.nonbonded_method)
+            self.system = self.parmed_structure.createSystem(nonbondedMethod=nonbonded_method_object,
+                                                             nonbondedCutoff=self.nonbonded_cutoff,
+                                                             constraints=constraints_object,
+                                                             removeCMMotion=False,
+                                                             hydrogenMass=4.0 * unit.amu if self.hmr else None)
+        else:
+            self.system = self.parmed_structure.createSystem(constraints=constraints_object,
+                                                             removeCMMotion=False,
+                                                             hydrogenMass=4.0 * unit.amu if self.hmr else None)
 
         if self.implicit_solvent_model:
             self.set_implicit_solvent_model(self.implicit_solvent_model, self.nonbonded_method, self.nonbonded_cutoff)
 
         if self.pressure:
-            if self.membrane:
-                print('setting membrane barostat')
-                barostat_force = mm.MonteCarloMembraneBarostat(1 * unit.bar, 200 * unit.bar * unit.nanometer,
-                                                               self.temperature,
-                                                               mm.MonteCarloMembraneBarostat.XYIsotropic,
-                                                               mm.MonteCarloMembraneBarostat.ZFree)
-            else:
-                print('setting barostat with pressure {}'.format(self.pressure))
-                barostat_force = mm.MonteCarloBarostat(self.pressure, self.temperature, 25)
-            self.system.addForce(barostat_force)
+            if self.nonbonded_method is not None:
 
+                if self.membrane:
+                    print('setting membrane barostat')
+                    barostat_force = mm.MonteCarloMembraneBarostat(1 * unit.bar, 200 * unit.bar * unit.nanometer,
+                                                                   self.temperature,
+                                                                   mm.MonteCarloMembraneBarostat.XYIsotropic,
+                                                                   mm.MonteCarloMembraneBarostat.ZFree)
+                    self.system.addForce(barostat_force)
+
+                else:
+                    print('setting barostat with pressure {}'.format(self.pressure))
+                    barostat_force = mm.MonteCarloBarostat(self.pressure, self.temperature, 25)
+                    self.system.addForce(barostat_force)
+            else:
+                print('cant use barostat with non-periodic system, overriding constant pressure condition')
 
         if self.atoms_to_freeze is not None:
             self.freeze_atom_selection(self.atoms_to_freeze)
@@ -506,7 +528,7 @@ class MDSimulation():
         if self.atoms_to_restrain is not None:
             self.set_positional_restraints(self.atoms_to_restrain, self.restraint_weight)
 
-        if self.restrain_COM:
+        if self.use_COM_restraint:
             atoms = select_atoms(self.parmed_structure, keyword_selection='protein ligand')
             self.restrain_COM(atoms, 20)
 
@@ -522,7 +544,7 @@ class MDSimulation():
             print('WARNING: saving an h5 file without the suffix .h5')
 
         # write hdf5 single frame
-        current_state = self.simulation.context.getState(getPositions=True,enforcePeriodicBox=True)
+        current_state = self.simulation.context.getState(getPositions=True,enforcePeriodicBox=self.wrap_state_coords)
         current_positions = current_state.getPositions()
 
         # mdtraj is set up to read from files more easily than memory so write a pdb to disk and
@@ -680,7 +702,7 @@ class MDSimulation():
         state = self.simulation.context.getState(getPositions=True,
                                                  getVelocities=True,
                                                  getEnergy=True,
-                                                 enforcePeriodicBox=True)
+                                                 enforcePeriodicBox=self.wrap_state_coords)
 
         current_positions = state.getPositions()
         current_velocities = state.getVelocities()
@@ -689,7 +711,8 @@ class MDSimulation():
         self.coordinates = np.array(current_positions.value_in_unit(unit.nanometer))
         self.parmed_structure.positions = current_positions
         self.parmed_structure.velocities = current_velocities
-        self.parmed_structure.box_vectors = current_box
+        if self.box_vectors is not None:
+            self.parmed_structure.box_vectors = current_box
         self.positions = current_positions
 
     def minimize(self, max_steps=None, save_h5=False):
@@ -699,7 +722,8 @@ class MDSimulation():
         :param save_h5: bool, optional
         :return: updated parmed_structure
         """
-        state = self.simulation.context.getState(getEnergy=True, enforcePeriodicBox=True)
+
+        state = self.simulation.context.getState(getEnergy=True, enforcePeriodicBox=self.wrap_state_coords)
         starting_energy = state.getPotentialEnergy()
 
         print('minimizing system with beginning energy: {}'.format(starting_energy))
@@ -713,7 +737,7 @@ class MDSimulation():
         #using enforce periodic box to make sure coordinates will reinitialize
         state = self.simulation.context.getState(getPositions=True,
                                                  getEnergy=True,
-                                                 enforcePeriodicBox=True)
+                                                 enforcePeriodicBox=self.wrap_state_coords)
         final_energy = state.getPotentialEnergy()
 
         # reset self.positions with current context positions
@@ -924,11 +948,12 @@ class MetadynamicSimulation(MDSimulation):
     def _build_custom_forces(self):
 
         #TODO it would probably be better to have a separate utilities call
-        if self.bias_directory is None:
+        if self.bias_directory is None or not os.path.exists(self.bias_directory):
             self.bias_directory = self.cwd
 
 
         # build funnel or dome restraints
+        #TODO try using a customcentroidbond force with flat bottom for this
         if self.dome:
             if self.membrane:
                 raise ValueError(
@@ -970,7 +995,7 @@ class MetadynamicSimulation(MDSimulation):
 
 
         if self.funnel:
-            print ('Warning! Funnel should only be used for correctly aligned structures.')
+            print ('Warning! Funnel should only be used for z-axis aligned structures.')
 
             # Now get the setup stage for placing funnel (otherwise coords may be wrapped)
             lig_atom_index = select_atoms(self.parmed_structure, keyword_selection='ligand')
@@ -985,12 +1010,12 @@ class MetadynamicSimulation(MDSimulation):
             print('prot ave zcoord used for funnel placement: {}'.format(np.mean(prot_positions, axis=0)[2]))
 
             # if the ligand is on the positive z-side use the z coordinate dimension
+            z_cone_apex = find_z_cone_apex(prot_positions,lig_positions)
+
             if np.mean(lig_positions, axis=0)[2] > np.mean(prot_positions, axis=0)[2]:
-                z_cone_apex = self.box_vectors[2][2].value_in_unit(unit.nanometer)
                 orient_factor = -1
             # else use the negative dimension
             else:
-                z_cone_apex = 0
                 orient_factor = 1
 
             r_cyl, center = find_radius_and_center(lig_positions, self.radius_scale_factor)
@@ -1001,7 +1026,6 @@ class MetadynamicSimulation(MDSimulation):
             print('Note! This funnel assumes that the ligand exit along the path is generally aligned ',
                   'with the z-axis')
 
-
             # the funnel shape depends on the radius and the x,y as function of z, needs to use periodicdistance()
             funnel_force = mm.CustomExternalForce(
                 'K*max(0, (r-radius))^2; r=sqrt(periodicdistance(x,y,z*0,x0,y0,zcc*0)^2); '
@@ -1307,10 +1331,15 @@ if __name__ == '__main__':
         if len(sys.argv[1:]) == 2:
             new_parmed = mdsim.run()
         else:
-            time = sys.argv[3]
+            time = float(sys.argv[3])
             new_parmed = mdsim.run(time=time * unit.nanosecond)
-        
-        new_parmed = mdsim.run()
 
         with open(new_parmed_file,'wb') as handle:
-            pickle.dump(new_parmed.__getstate__(), handle)
\ No newline at end of file
+            pickle.dump(new_parmed.__getstate__(), handle)
+
+        #pick out and rename bias_file for metaD sims
+        pattern = re.compile('bias_(.*)_(.*)\.npy')
+        matches = [pattern.match(filename) for filename in os.listdir(mdsim.cwd) if pattern.match(filename)!=None]
+        if len(matches) > 0:
+            bias_file = os.path.join(mdsim.cwd, matches[-1].string)
+            shutil.move(bias_file, "bias.npy")
